/***************************************************************
From the book "Motif Programming, The Essentials...and More"
by Marshall Brain.
Published by Digital Press, ISBN 1-55558-089-0.
To order the book, call 1-800-DIGITAL and ask for EY-J816E-DP.

Copyright 1992, by Digital Equipment Corp.

This code demonstrates the use of accelerators in a menu.
***************************************************************/

/* accelerator.c */

#include <Xm/Xm.h>
#include <Xm/Label.h> 
#include <Xm/Form.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>

XtAppContext context;
XmStringCharSet char_set=XmSTRING_DEFAULT_CHARSET;

Widget toplevel, form, label, menu_bar;
Widget file_menu;
Widget open_item;
Widget close_item;
Widget quit_item;
Widget edit_menu;
Widget copy_item;
Widget cut_item;
Widget paste_item;

void menuCB(w,client_data,call_data)
    Widget w;
    char *client_data;
    XmAnyCallbackStruct *call_data;
/* callback routine used for all menus */
{
    printf("%s\n",client_data);
    if (strcmp(client_data,"Quit")==0) /* if quit seen, then exit */
        exit(0);
}

Widget make_menu_item(item_name,client_data,menu)
    char *item_name;
    caddr_t client_data;
    Widget menu;
/* adds an item into a menu. */ 
{
    int ac;
    Arg al[10];
    Widget item;

    ac = 0;
    XtSetArg(al[ac],XmNlabelString,
        XmStringCreateLtoR(item_name,char_set)); ac++;  
    item=XmCreatePushButton(menu,item_name,al,ac);
    XtManageChild(item);
    XtAddCallback(item,XmNactivateCallback,menuCB,client_data);  
    XtSetSensitive(item,True);
    return(item);
}

Widget make_menu(menu_name,menu_bar)
    char *menu_name; 
    Widget menu_bar;
/* creates a menu on the menu bar */
{
    int ac;
    Arg al[10];
    Widget menu, cascade;

    menu=XmCreatePulldownMenu(menu_bar,menu_name,NULL,0);
    ac=0;
    XtSetArg (al[ac],XmNsubMenuId, menu); ac++;
    XtSetArg(al[ac],XmNlabelString,
        XmStringCreateLtoR(menu_name,char_set)); ac++;
    cascade=XmCreateCascadeButton(menu_bar,menu_name,al,ac);  
    XtManageChild(cascade); 
    return(menu);
}

void add_accelerator(w, acc_text, key)
    Widget w;
    char *acc_text;
    char *key;
/* adds an accelerator to a menu option. */
{
    int ac;
    Arg al[10];

    ac=0;
    XtSetArg(al[ac],XmNacceleratorText,
        XmStringCreate(acc_text,XmSTRING_DEFAULT_CHARSET)); ac++;
    XtSetArg(al[ac],XmNaccelerator,key); ac++;
    XtSetValues(w,al,ac);
}

void create_menus(menu_bar)
    Widget menu_bar;
/* creates all the menus for this program */
{
    /* create the file menu */
    file_menu=make_menu("File",menu_bar);
    open_item=make_menu_item("Open","Open selected",file_menu);
    add_accelerator(open_item,"meta+o","Meta<Key>o:");
    close_item=make_menu_item("Close","Close selected",file_menu);
    quit_item=make_menu_item("Quit","Quit",file_menu);

    /* create the edit menu */
    edit_menu=make_menu("Edit",menu_bar);
    copy_item=make_menu_item("Copy","Copy selected",edit_menu);
    cut_item=make_menu_item("Cut","Cut Selected",edit_menu);
    paste_item=make_menu_item("Paste","Paste Selected",edit_menu);
}

void main(argc,argv)
    int argc; 
    char *argv[];
{
    Arg al[10];
    int ac;

    /* create the toplevel shell */
    toplevel = XtAppInitialize(&context,"",NULL,0,&argc,argv,NULL,NULL,0);

    /* resize the window */
    ac=0;
    XtSetArg(al[ac],XmNheight,200); ac++;
    XtSetArg(al[ac],XmNwidth,200); ac++;
    XtSetValues(toplevel,al,ac);

    /* create a form widget */
    ac=0;
    form=XmCreateForm(toplevel,"form",al,ac);
    XtManageChild(form);

    /* create a label widget */
    ac=0;
    XtSetArg(al[ac],XmNlabelString,
        XmStringCreate("I'm a label", char_set)); ac++;
    label=XmCreateLabel(form,"label",al,ac);
    XtManageChild(label);

    /* create the menu bar */
    ac=0;
    menu_bar=XmCreateMenuBar(form,"menu_bar",al,ac);
    XtManageChild(menu_bar);

    /* attach the menu bar to the form */
    ac=0;
    XtSetArg(al[ac],XmNtopAttachment,XmATTACH_FORM); ac++;
    XtSetArg(al[ac],XmNrightAttachment,XmATTACH_FORM); ac++;
    XtSetArg(al[ac],XmNleftAttachment,XmATTACH_FORM); ac++;
    XtSetValues(menu_bar,al,ac);

    /* attach the label to the form */
    ac=0;
    XtSetArg(al[ac],XmNtopAttachment,XmATTACH_WIDGET); ac++;
    XtSetArg(al[ac],XmNtopWidget,menu_bar); ac++;
    XtSetArg(al[ac],XmNrightAttachment,XmATTACH_FORM); ac++;
    XtSetArg(al[ac],XmNleftAttachment,XmATTACH_FORM); ac++;
    XtSetArg(al[ac],XmNbottomAttachment,XmATTACH_FORM); ac++;
    XtSetValues(label,al,ac);

    create_menus(menu_bar);

    XtRealizeWidget(toplevel);
    XtAppMainLoop(context);
}
